#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include"SimpleImage.h" /* 授業で使ってきた画像クラス */
#include"BMPIO.h" /* BMP画像の入出力関数 */
#include"FDCT.h" /* 高速フーリエ変換(正確にはDCTのみ実装なので「高速離散コサイン変換」) */


/* 周波数画像を保存するための関数 */
void saveSpectrum(FDCT *fR,FDCT *fG,FDCT *fB,char *filename){
  BMPIO *mybmp = new BMPIO();
  Image *R = new Image(fR->in->sx,fR->in->sy);
  Image *G = new Image(fG->in->sx,fG->in->sy);
  Image *B = new Image(fB->in->sx,fB->in->sy);
  
  fR->toSpectrum(R);
  fG->toSpectrum(G);
  fB->toSpectrum(B);
  
  
  mybmp->saveBMP(R,G,B,filename);
  
  delete R;
  delete G;
  delete B;
  delete mybmp;
}


//g++ -O3 testFDCT.cxx -o testFDCT -lm
//./testFDCT 入力画像 出力画像 カットする周波数のしきい値(int)

int main(int argc,char *argv[]){

  /* 変数の宣言 */
  BMPIO *mybmp = new BMPIO();
  Image *R= NULL;
  Image *G= NULL;
  Image *B= NULL;
  int sx,sy;
  
   /* argv[1]で与えられるファイル名の画像サイズの読み込み */
  if(mybmp->readBMPSize(&sx,&sy,argv[1])){
    /* メモリ確保 */
    R = new Image(sx,sy);
    G = new Image(sx,sy);
    B = new Image(sx,sy);
    
     /* argv[1]で与えられるファイル名の画像の読み込み */
    mybmp->readBMP(R,G,B,argv[1]);
    
  }else{
    delete mybmp;
    printf("Unsupported BMP format !\n");
    return 1;
  }
  

  /* FFTクラスの宣言：注意：各画像毎に作成する 
     画像の大きさが2の乗数でないとFFTは摘要出来ないためFDCTクラス内では一旦入力画像より大きな(2の乗数)×(2の乗数)の画像に変換して、
     FDCTクラス内のImage *inを作成している。
     縦：fft_R->in->sy
     横：fft_R->in->sx
     中身の画像:fft_R->in->img[][]
     
   */
  FDCT *fft_R = new FDCT(R);
  FDCT *fft_G = new FDCT(G);
  FDCT *fft_B = new FDCT(B);
  


  /* 高速DCT 順変換 */
  fft_R->DCT();
  fft_G->DCT();
  fft_B->DCT();
  char tmpout1[1000];
  sprintf(tmpout1,"%s_spectrum.bmp",argv[2]);
  saveSpectrum(fft_R,fft_G,fft_B,tmpout1);
  int i,j;
  /* targetより上の周波数成分を円状にカット(ゼロにする): ロウパスフィルタ */
  int target=atoi(argv[3]);
  
  
  /* 周波数画像のループ 
     縦：fft_R->in->sy
     横：fft_R->in->sx
     でループを回している事に注意
     */
 
  double tr = ((double)(target));
  for(i=0;i<fft_R->in->sy;i++)
    for(j=0;j<fft_R->in->sx;j++){
      double radi = sqrt((double)(i*i+j*j));
      if(radi>=tr){
	fft_R->in->img[i][j]=0.0;
	fft_G->in->img[i][j]=0.0;
	fft_B->in->img[i][j]=0.0;
      }
    }
 
  char tmpout2[1000];
  sprintf(tmpout2,"%s_smooth_spectrum.bmp",argv[2]);
  saveSpectrum(fft_R,fft_G,fft_B,tmpout2);


  /* DCT 逆変換 
     変換後にR,G,Bの大きさの画像にコピーしている

   */
  fft_R->InverseDCT(R);
  fft_G->InverseDCT(G);
  fft_B->InverseDCT(B);
  
  /* 逆変換が8bit:0-255になるとは限らないので*/
  
  for(i=0;i<R->sy;i++)
    for(j=0;j<R->sx;j++){
      if(R->img[i][j]<0.0)R->img[i][j]=0.0;
      if(R->img[i][j]>255.0)R->img[i][j]=255.0;
      if(G->img[i][j]<0.0)G->img[i][j]=0.0;
      if(G->img[i][j]>255.0)G->img[i][j]=255.0;
      if(B->img[i][j]<0.0)B->img[i][j]=0.0;
      if(B->img[i][j]>255.0)B->img[i][j]=255.0;
    }
  
  /* 画像に保存 */

  char tmpout8[1000];
  sprintf(tmpout8,"%s_smooth.bmp",argv[2]);
  mybmp->saveBMP(R,G,B,tmpout8);
  
  

  /* メモリ開放 */
  
  delete fft_R;
  delete fft_G;
  delete fft_B;
  
  
  delete mybmp;
  if(R!=NULL)delete R;
  if(G!=NULL)delete G;
  if(B!=NULL)delete B;
 
  return 0;
}

