#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include"SimpleImage.h" /* 授業で使ってきた画像クラス */
#include"BMPIO.h" /* BMP画像の入出力関数 */
#include"FDCT.h" /* 高速フーリエ変換(正確にはDCTのみ実装なので「高速離散コサイン変換」) */


/* 周波数画像を保存するための関数 */
void saveSpectrum(FDCT *fR,FDCT *fG,FDCT *fB,char *filename){
  BMPIO *mybmp = new BMPIO();
  Image *R = new Image(fR->in->sx,fR->in->sy);
  Image *G = new Image(fG->in->sx,fG->in->sy);
  Image *B = new Image(fB->in->sx,fB->in->sy);
  
  fR->toSpectrum(R);
  fG->toSpectrum(G);
  fB->toSpectrum(B);
  
  
  mybmp->saveBMP(R,G,B,filename);
  
  delete R;
  delete G;
  delete B;
  delete mybmp;
}

/* R,G,Bの最大・最小を使った正規化 */
void Normalize(Image *R,Image *G,Image *B){
  double max,min;
  max= min = R->img[0][0];
  int i,j;
  
  for(i=0;i<R->sy;i++)
    for(j=0;j<R->sx;j++){
      if(max<R->img[i][j])max=R->img[i][j];
      if(max<G->img[i][j])max=G->img[i][j];
      if(max<B->img[i][j])max=B->img[i][j];
      if(min>R->img[i][j])min=R->img[i][j];
      if(min>G->img[i][j])min=G->img[i][j];
      if(min>B->img[i][j])min=B->img[i][j];
    }
  double range = fabs(max-min);if(range<=0.0)range=1.0;
   for(i=0;i<R->sy;i++)
    for(j=0;j<R->sx;j++){
      double val = 255.0*(R->img[i][j]-min)/range;if(val<0.0)val=0.0;if(val>255.0)val=255.0;
      R->img[i][j] = val;
      val = 255.0*(G->img[i][j]-min)/range;if(val<0.0)val=0.0;if(val>255.0)val=255.0;
      G->img[i][j] = val;
      val = 255.0*(B->img[i][j]-min)/range;if(val<0.0)val=0.0;if(val>255.0)val=255.0;
      B->img[i][j] = val;
    }


}

//g++ -O3 testFrequency.cxx -o testFrequency -lm
//./testFFrequency 入力画像 出力画像 バンドパスフィルタのしきい値(int)

int main(int argc,char *argv[]){

  /* 変数の宣言 */
  BMPIO *mybmp = new BMPIO();
  Image *R= NULL;
  Image *G= NULL;
  Image *B= NULL;
  int sx,sy;
  
   /* argv[1]で与えられるファイル名の画像サイズの読み込み */
  if(mybmp->readBMPSize(&sx,&sy,argv[1])){
    /* メモリ確保 */
    R = new Image(sx,sy);
    G = new Image(sx,sy);
    B = new Image(sx,sy);
    
     /* argv[1]で与えられるファイル名の画像の読み込み */
    mybmp->readBMP(R,G,B,argv[1]);
    
  }else{
    delete mybmp;
    printf("Unsupported BMP format !\n");
    return 1;
  }
  

  /* FFTクラスの宣言：注意：各画像毎に作成する 
     画像の大きさが2の乗数でないとFFTは摘要出来ないためFDCTクラス内では一旦入力画像より大きな(2の乗数)×(2の乗数)の画像に変換して、
     FDCTクラス内のImage *inを作成している。
     縦：fft_R->in->sy
     横：fft_R->in->sx
     中身の画像:fft_R->in->img[][]
     
   */
  FDCT *fft_R = new FDCT(R);
  FDCT *fft_G = new FDCT(G);
  FDCT *fft_B = new FDCT(B);
  
  
  /* 高速DCT 順変換 */
  fft_R->DCT();
  fft_G->DCT();
  fft_B->DCT();
  
  /* 作業用のテンポラリー画像 
     fft_R,fft_G,fft_Bは特定周波数を取り出す操作をするので、オリジナルの周波数を保存しておく
   
   */
  Image *fft_temp_R = new Image(fft_R->in->sx,fft_R->in->sy);
  Image *fft_temp_G = new Image(fft_G->in->sx,fft_G->in->sy);
  Image *fft_temp_B = new Image(fft_B->in->sx,fft_B->in->sy);
  fft_R->CopyTo(fft_temp_R);
  fft_G->CopyTo(fft_temp_G);
  fft_B->CopyTo(fft_temp_B);
  
  /* オリジナルの周波数画像の保存 */

  char tmpout1[1000];
  sprintf(tmpout1,"%s_spectrum.bmp",argv[2]);
  saveSpectrum(fft_R,fft_G,fft_B,tmpout1);
  int i,j;



  /* targetからドーナッツ状に半分ずつの厚みの周波数成分を抽出: バンドパスフィルタ */
  int target=atoi(argv[3]);
  
  /* 半径1ピクセルまで続けるループ*/
  
  int outer=target;
  int inner=target/2;
  while(inner>=1){
    
    /* 周波数画像のループ 
     縦：fft_R->in->sy
     横：fft_R->in->sx
     でループを回している事に注意
     */
    double tr1 = ((double)(outer));
    double tr2 = ((double)(inner));
    
    for(i=0;i<fft_R->in->sy;i++)
      for(j=0;j<fft_R->in->sx;j++){
	double radi = sqrt((double)(i*i+j*j));
	/* ドーナツ状に周波数を残す：バンドパスフィルタ */
	if(radi>tr1||radi<tr2){
	  fft_R->in->img[i][j]=0.0;
	  fft_G->in->img[i][j]=0.0;
	  fft_B->in->img[i][j]=0.0;
	}
      }
    char tmpout2[1000];
    sprintf(tmpout2,"%s_frequency_%02d_%02d.bmp",argv[2],inner,outer);
    saveSpectrum(fft_R,fft_G,fft_B,tmpout2);
    
    
    /* DCT 逆変換 
       変換後にR,G,Bの大きさの画像にコピーしている
       
    */
    fft_R->InverseDCT(R);
    fft_G->InverseDCT(G);
    fft_B->InverseDCT(B);
    
    /* 周波数成分画像は逆変換が正や8bit:0-255になるとは限らないので0-255に正規化 */
    Normalize(R,G,B);
    
    /* 画像に保存 */
    char tmpout8[1000];
    sprintf(tmpout8,"%s_bandpath_%02d_%02d.bmp",argv[2],inner,outer);
    mybmp->saveBMP(R,G,B,tmpout8);
    
    
    /* 次の周波数領域を半分の大きさのドーナツ状に設定する */
    outer=inner;
    inner /= 2;
    
    
    
    /* 保存先から次の周波数領域の操作のために保存しておいたオリジナルの周波数画像をコピーする */
    fft_R->CopyFrom(fft_temp_R);
    fft_G->CopyFrom(fft_temp_G);
    fft_B->CopyFrom(fft_temp_B);
    
  }
  /* 最後にtarget以上の高周波成分画像を生成*/
  
   for(i=0;i<fft_R->in->sy;i++)
      for(j=0;j<fft_R->in->sx;j++){
	double radi = sqrt((double)(i*i+j*j));
	/* ドーナツ状に周波数を残す：バンドパスフィルタ */
	if(radi<target){
	  fft_R->in->img[i][j]=0.0;
	  fft_G->in->img[i][j]=0.0;
	  fft_B->in->img[i][j]=0.0;
	}
      }
    char tmpout2[1000];
    sprintf(tmpout2,"%s_frequency_%02d_End.bmp",argv[2],target);
    saveSpectrum(fft_R,fft_G,fft_B,tmpout2);
    fft_R->InverseDCT(R);
    fft_G->InverseDCT(G);
    fft_B->InverseDCT(B);
    Normalize(R,G,B);
    char tmpout8[1000];
    sprintf(tmpout8,"%s_bandpath_%02d_End.bmp",argv[2],target);
    mybmp->saveBMP(R,G,B,tmpout8);
    
  
 
  
  

  /* メモリ開放 */
  
  delete fft_R;
  delete fft_G;
  delete fft_B;
  delete fft_temp_R;
  delete fft_temp_G;
  delete fft_temp_B;
  
  
  delete mybmp;
  if(R!=NULL)delete R;
  if(G!=NULL)delete G;
  if(B!=NULL)delete B;
 
  return 0;
}

