#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include"SimpleImage.h" /* 授業で使ってきた画像クラス */
#include"BMPIO.h" /* BMP画像の入出力関数 */
#include"FDCT.h" /* 高速フーリエ変換(正確にはDCTのみ実装なので「高速離散コサイン変換」) */


/* 周波数画像を保存するための関数 */
void saveSpectrum(FDCT *fR,FDCT *fG,FDCT *fB,char *filename){
  BMPIO *mybmp = new BMPIO();
  Image *R = new Image(fR->in->sx,fR->in->sy);
  Image *G = new Image(fG->in->sx,fG->in->sy);
  Image *B = new Image(fB->in->sx,fB->in->sy);
  
  fR->toSpectrum(R);
  fG->toSpectrum(G);
  fB->toSpectrum(B);
  
  
  mybmp->saveBMP(R,G,B,filename);
  
  delete R;
  delete G;
  delete B;
  delete mybmp;
}

/* ガウス関数を左上を原点に作成する関数 */
void makeGauss(Image *in,double sigma){
  int i,j;
  double sum=0.0;
  double SIG=1.0/(2.0*sigma*sigma);
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      double val = exp(-((double)(i*i+j*j))*SIG);
      in->img[i][j] = val;
    }
}

void saveGauss(Image *in,char *filename){
  Image *tmp = new Image(in->sx,in->sy);
  BMPIO *mybmp = new BMPIO();
  int i,j;
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      tmp->img[i][j] = 255.0*in->img[i][j];
      if(tmp->img[i][j]>255.0)tmp->img[i][j]=255.0;
    }
  mybmp->saveBMP(tmp,tmp,tmp,filename);
  
  delete mybmp;
  delete tmp;
}




//g++ -O3 testGauss_FDCT.cxx -o testGauss_FDCT -lm
//./testGauss_FDCT 入力画像 出力画像 ガウス関数の標準偏差(double)

int main(int argc,char *argv[]){

  /* 変数の宣言 */
  BMPIO *mybmp = new BMPIO();
  Image *R= NULL;
  Image *G= NULL;
  Image *B= NULL;
  int sx,sy;
  
   /* argv[1]で与えられるファイル名の画像サイズの読み込み */
  if(mybmp->readBMPSize(&sx,&sy,argv[1])){
    /* メモリ確保 */
    R = new Image(sx,sy);
    G = new Image(sx,sy);
    B = new Image(sx,sy);
    
     /* argv[1]で与えられるファイル名の画像の読み込み */
    mybmp->readBMP(R,G,B,argv[1]);
    
  }else{
    delete mybmp;
    printf("Unsupported BMP format !\n");
    return 1;
  }
  

  /* FFTクラスの宣言：注意：各画像毎に作成する 
     画像の大きさが2の乗数でないとFFTは摘要出来ないためFDCTクラス内では一旦入力画像より大きな(2の乗数)×(2の乗数)の画像に変換して、
     FDCTクラス内のImage *inを作成している。
     縦：fft_R->in->sy
     横：fft_R->in->sx
     中身の画像:fft_R->in->img[][]
     
   */
  FDCT *fft_R = new FDCT(R);
  FDCT *fft_G = new FDCT(G);
  FDCT *fft_B = new FDCT(B);
  


  /* 高速DCT 順変換 */
  fft_R->DCT();
  fft_G->DCT();
  fft_B->DCT();
  
  /* 周波数画像の保存 */
  char tmpout1[1000];
  sprintf(tmpout1,"%s_spectrum.bmp",argv[2]);
  saveSpectrum(fft_R,fft_G,fft_B,tmpout1);
  
  
  
  /* ガウス画像の作成 */
  double sigma=atof(argv[3]);//ゼロより大きな浮動小数点
  Image *gauss = new Image(fft_R->in->sx,fft_R->in->sy);
  makeGauss(gauss,sigma);
 

  
  /* ガウス関数の保存 */
  char tmpout3[1000];
  sprintf(tmpout3,"%s_gauss.bmp",argv[2]);
  saveGauss(gauss,tmpout3);
  
 
  
  
  
  /* 周波数画像のループ：畳み込み=周波数領域ではフーリエ変換された二つの関数の積となる 
     縦：fft_R->in->sy
     横：fft_R->in->sx
     でループを回している事に注意
     */
  int i,j;
  for(i=0;i<fft_R->in->sy;i++)
    for(j=0;j<fft_R->in->sx;j++){
      
      fft_R->in->img[i][j]*=gauss->img[i][j];
      fft_G->in->img[i][j]*=gauss->img[i][j];
      fft_B->in->img[i][j]*=gauss->img[i][j];
      
    }
 
  char tmpout2[1000];
  sprintf(tmpout2,"%s_gaussian_spectrum.bmp",argv[2]);
  saveSpectrum(fft_R,fft_G,fft_B,tmpout2);


  /* DCT 逆変換 
     変換後にR,G,Bの大きさの画像にコピーしている

   */
  fft_R->InverseDCT(R);
  fft_G->InverseDCT(G);
  fft_B->InverseDCT(B);
  
  /* 逆変換が8bit:0-255になるとは限らないので*/
  
  for(i=0;i<R->sy;i++)
    for(j=0;j<R->sx;j++){
      if(R->img[i][j]<0.0)R->img[i][j]=0.0;
      if(R->img[i][j]>255.0)R->img[i][j]=255.0;
      if(G->img[i][j]<0.0)G->img[i][j]=0.0;
      if(G->img[i][j]>255.0)G->img[i][j]=255.0;
      if(B->img[i][j]<0.0)B->img[i][j]=0.0;
      if(B->img[i][j]>255.0)B->img[i][j]=255.0;
    }
  
  /* 画像に保存 */

  char tmpout8[1000];
  sprintf(tmpout8,"%s_gaussian.bmp",argv[2]);
  mybmp->saveBMP(R,G,B,tmpout8);
  
  

  /* メモリ開放 */
  
  delete fft_R;
  delete fft_G;
  delete fft_B;
  delete gauss;
  delete mybmp;
  if(R!=NULL)delete R;
  if(G!=NULL)delete G;
  if(B!=NULL)delete B;
 
  return 0;
}

